<?php

/**
 * Z nameranych dat vyrata BER, SER a BlockER a spolu s vyparsovanym SNR ulozi do DIR_OUT
 * 
 * FEATURES:
 * 	statistika posledneho neuplneho bloku sa nevykonava
 * 	statistika posledneho neuplneho symbolu sa nevykonava [ak je neparny pocet bitov]
 * 	SNR sa vytiahne z .txt suboru automaticky
 * 	
 * RESULTS:
 * 	pre kazdy vstupny subor sa vytvori vystupny subor s BER, SER, BlockER so zhodnym nazvom do DIR_OUT
 * 	vysledny subor obsahuje  SNR a 1 z {BER, SER, BlockER} v stlpcoch pre pouzitie v programe MATLAB. Nazov je odvodeny od pouzitej metody chybovosti [stat_ber | stat_ser | stat_blocker]
 * 
 * @author Matus Matula
 * @copyright WiTFi Team
 * @version 0.1
 *
 */

//echo ignore_user_abort(false);
//die();

define('NL', "\r\n"); // NEW LINE do file
define('NL2', "<br />\n"); // NEW LINE na stranke
define('BITS_PER_SYMBOL', '2');	//	pocet bitov na symbol
define('BITS_PER_BLOCK', '512');	//	pocet bitov na blok
define('FIXABLE_BITS_PER_BLOCK', '12');	//	pocet bitov, kt. dokazeme opravit v ramci bloku
define('PRECISION', '2');	//	pocet desatinnych miest pre chybovost
define('DEBUG', '0'); // nastavit debugovaci mod
define('DIR_IN', 'e:/timovy_projekt/in/'); // path k vstupnym datam
define('DIR_OUT', 'e:/timovy_projekt/out/'); // path k vystupnym datam

//get_final_stat();
//die();

//	nacitame mena suborov s datami do pola
$files = load_dir(DIR_IN);

//	robim statistiku vsetkych naraz
foreach ($files as $filename) {
	
	//	uzivatel klikne STOP
	if (connection_aborted() == 1) {
		file_put_contents(DIR_OUT . 'test.txt', 'user abort' . NL . 'ds');
	}
	
	//	cesta k suboru
	$path = DIR_IN . $filename;
	
	//	ak subor existuje
	if (file_exists($path)) {
		
		//	nacitame data do stringu a orezeme novy riadok [ak tam je]
		$data = rtrim(file_get_contents($path));
		
		//	inicializacia citaca chyb
		$bit_block_errors = $block_errors = $bit_errors = $symbol_errors = 0; // bit-block pocet chybnych bitov v bloku .. pouzita iba na docasne vypocty
		
		//	dlzka retazca = pocet bitov
		$bits_count = strlen($data);
		
		//	pocet symbolov .. ak mi ostane posledny nesparovany bit, tak to NEpovazujem za symbol
		$symbols_count = floor($bits_count / BITS_PER_SYMBOL);
		
		//	pocet blokov .. posledny neuplny blok NEpovazujem za blok
		$blocks_count = floor($bits_count / BITS_PER_BLOCK);
		
		for ($i=0; $i<$bits_count; $i++) {
			
			//	zaciname novy blok => reset counter
			if (($i % BITS_PER_BLOCK) == 0) {
				//	ak je v bloku viac chyb, ako dokazeme opravit => chybny blok
				if ($bit_block_errors > FIXABLE_BITS_PER_BLOCK) {
					$block_errors++;
				}
				
				//	reset counter
				$bit_block_errors = 0;
			}
			
			//	chybny bit ? => zapocitaj bitovu a bitovu v blokoch
			if ($data[$i] === '1') {
				$bit_errors++;
				$bit_block_errors++;
			}
			
			//	chybny symbol ?
			//	kazdy neparny bit kontrolujem .. predidem tak moznej chybe na konci suboru, keby uz nebol dalsi byt [posledny do symbolu]
			if (($i % 2) == 1) {
				// ak tento alebo predchadzajuci je zly, symbol je zly
				if ($data[$i] === '1' or $data[$i-1] === '1') {
					$symbol_errors++;
				}
			}
		}

		if (DEBUG) {
			echo $bit_errors . NL2;
			echo $symbol_errors . NL2;
//			echo $bits_count;
//			echo $symbols_count;
		}
		
		$ber = $bit_errors / $bits_count;
		$ser = $symbol_errors / $symbols_count;
		$blocker = $block_errors / $blocks_count;
		
		echo 'subor ' . $filename . ' ---> 
			BER: ' . round($ber * 100, PRECISION) . '% 
			SER: ' . round($ser * 100, PRECISION) . '%
			BlockER: ' . round($blocker * 100, PRECISION) . '%
			' . NL2;
		
		flush(); // vyvrhnem buffer => tym padom apache skontroluje spojenie [ci nahodou user nezrusil]
		
		$data_out = 'BER: ' . round($ber, PRECISION * 2) . NL . 
					'SER: ' . round($ser, PRECISION * 2) . NL . 
					'BlockER: ' . round($blocker, PRECISION * 2);
		
		file_put_contents(DIR_OUT . $filename, $data_out);
		
			/*	NACITAM SNR Z .txt SUBORU	*/
		//	cesta k suboru so statistickym zhrnutim .. odtial chcem SNR
		$path_stat = $path . '.txt';
		
		//	ak subor existuje
		if (file_exists($path_stat)) {
		
			//	nacitame data do stringu a orezeme novy riadok [ak tam je]
			$data = rtrim(file_get_contents($path_stat));
			
			//	najdem poslednu medzeru = zaciatok cisla SNR
			$snr_pos = strrpos($data, " ");
			
			//	citam az do konca == SNR
			$snr = substr($data, $snr_pos+1);
		}
			/*	NACITAM SNR Z .txt SUBORU END	*/
		
		// pridam vyhodnotene statisticke udaje do suboru
		file_put_contents(DIR_OUT . 'stat_ber.txt', $snr . ' ' . $ber . NL, FILE_APPEND);
		file_put_contents(DIR_OUT . 'stat_ser.txt', $snr . ' ' . $ser . NL, FILE_APPEND);
		file_put_contents(DIR_OUT . 'stat_blocker.txt', $snr . ' ' . $blocker . NL, FILE_APPEND);
		
		
	} else {
		echo 'subor neexistuje';
	}
}

// pouzitie je iba teraz dodatocne a docasne, kedze statistiku uz mam, tak kvoli koncovemu vyhodnoteniu to netreba znova spustat
function get_final_stat() {
	
	$files = load_dir(DIR_OUT);
	
	//	robim statistiku vsetkych naraz
	foreach ($files as $filename) {
		
		//	cesta k suboru
		$path = DIR_OUT . $filename;
		
		//	ak subor existuje
		if (file_exists($path)) {
			
			//	nacitame data do stringu a orezeme novy riadok [ak tam je]
			$data = file($path);
			$ber = substr(rtrim($data[0]), 4);
			$ser = substr(rtrim($data[1]), 4);
			$blocker = substr($data[2], 8);
		}
			/*	NACITAM SNR Z .txt SUBORU	*/
		//	cesta k suboru so statistickym zhrnutim .. odtial chcem SNR
		$path_stat = DIR_IN . $filename . '.txt';
			
		//	ak subor existuje
		if (file_exists($path_stat)) {
		
			//	nacitame data do stringu a orezeme novy riadok [ak tam je]
			$data = rtrim(file_get_contents($path_stat));

			//	najdem poslednu medzeru = zaciatok cisla SNR
			$snr_pos = strrpos($data, " ");
			
			//	citam az do konca == SNR
			$snr = substr($data, $snr_pos+1);
		}
			/*	NACITAM SNR Z .txt SUBORU END	*/
		
		// pridam vyhodnotene statisticke udaje do suboru
		file_put_contents(DIR_OUT . 'stat_ber.txt', $snr . ' ' . $ber . NL, FILE_APPEND);
		file_put_contents(DIR_OUT . 'stat_ser.txt', $snr . ' ' . $ser . NL, FILE_APPEND);
		file_put_contents(DIR_OUT . 'stat_blocker.txt', $snr . ' ' . $blocker . NL, FILE_APPEND);
	
	}

}


function load_dir($dir) {
	
 	if (($dir = @opendir($dir)) != false) {
     	while (false !== ($file = readdir($dir))) {
     		//	nacitame iba dumpy s datami
     		if (strpos($file, '.txt') === false and $file !== '.' and $file !== '..') {
	         	$files[] = $file;
	         	
	         	if (DEBUG) {
		         	break; // zatial iba 1 vratim
	         	}
     		}
    	}
    	closedir($dir);
  	}
 
  	return $files;
}